/*
 * Copyright 2002-2011 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.bank.config.xml;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.instanceOf;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

import org.aopalliance.aop.Advice;
import org.junit.Before;
import org.junit.Test;
import org.springframework.aop.Advisor;
import org.springframework.aop.framework.Advised;
import org.springframework.context.support.GenericXmlApplicationContext;
import org.springframework.transaction.interceptor.TransactionInterceptor;

import com.bank.domain.InsufficientFundsException;
import com.bank.repository.AccountRepository;
import com.bank.service.TransferService;

public class IntegrationTests {

	private TransferService transferService;
	private AccountRepository accountRepository;

	@Before
	public void setUp() {
		GenericXmlApplicationContext ctx = new GenericXmlApplicationContext();
		ctx.getEnvironment().setActiveProfiles("dev");
		ctx.load("classpath:/com/bank/config/xml/transfer-service-config.xml");
		ctx.refresh();

		transferService = ctx.getBean(TransferService.class);
		accountRepository = ctx.getBean(AccountRepository.class);
	}

	@Test
	public void transferTenDollars() throws InsufficientFundsException {
		assertThat(accountRepository.findById("A123").getBalance(), equalTo(100.00));
		assertThat(accountRepository.findById("C456").getBalance(), equalTo(0.00));

		transferService.transfer(10.00, "A123", "C456");

		assertThat(accountRepository.findById("A123").getBalance(), equalTo(90.00));
		assertThat(accountRepository.findById("C456").getBalance(), equalTo(10.00));
	}

	@Test
	public void transactionalAdviceIsApplied() {
		assertThat("expected transferService to be a Spring AOP proxy",
				transferService, instanceOf(Advised.class));

		boolean hasTxAdvice = false;
		for (Advisor advisor : ((Advised)transferService).getAdvisors()) {
			Advice advice = advisor.getAdvice();
			if (advice instanceof TransactionInterceptor) {
				hasTxAdvice = true;
				break;
			}
		}

		assertThat("expected transferService to be advised with a TransactionInterceptor",
				hasTxAdvice, is(true));
	}

}
